@@ -3,12 +3,12 @@ import fabio
 import datetime,os,copy
 from py4xs.mask import Mask
 from py4xs.local import ExpPara
-from py4xs.utils import calc_avg
+from py4xs.utils import calc_avg,auto_clim
 import pylab as plt
 import matplotlib as mpl
-from matplotlib.colors import LogNorm
 from enum import Enum 
 from PIL import Image
+#import fast_histogram as fh
 
 class DataType(Enum):
     det = 1
@@ -30,6 +30,24 @@ def get_bin_edges(grid):
         v0 = v1
     return np.asarray(edges)
 
+def get_bins_range(qgrid, tol=0.01):
+    """ break down non-uniform qgrid into multiple evenly spaced ranges 
+        [[[min1, max1], N1], [[min2, max2], N2], ...]
+        there will be a gap between neighonring ranges
+    """
+    bin_range = []
+    while True:
+        qmin = qgrid[0]
+        dq = qgrid[1]-qgrid[0]
+        pq = qmin+dq*np.arange(len(qgrid))
+        N = len(qgrid[np.fabs(qgrid-pq)<dq*tol])
+        bin_range.append([[qmin-0.5*dq, qmin+(N-0.5)*dq], N])
+        if N==len(qgrid):
+            break
+        qgrid = qgrid[N:]
+        
+    return bin_range
+
 def round_by_stepsize(v, ss):
     prec = -int(np.floor(np.log10(np.fabs(ss))))
     if prec<1:
@@ -72,15 +90,27 @@ def grid_labels(grid, N=3, step_tol=0.2):
         gpvalues.append(gp[-1])
         gplabels.append(round_by_stepsize(gp[-1],ss))
     
+    # on a 2D plot, the pixel size is finite, the extent of the axis goes from -0.5 to n-0.5
+    # it appears that the only way to align the ticks on a twin axis is to have a blank at the end
+    n = len(grid)
+    gpindex = np.append(0.5+np.array(gpindex), n)
+    gplabels.append('')
+    
     return gpindex,gpvalues,gplabels
 
+def histogram2d(x, y, range, bins, weights):
+    return np.histogram2d(x, y, range=range, bins=bins, weights=weights)
+    #return fh.histogram2d(x, y, range=range, bins=bins, weights=weights)
+
 class MatrixWithCoords:
     # 2D data with coordinates
     d = None
     xc = None
     xc_label = None
+    xc_prec = 3
     yc = None 
     yc_label = None
+    yc_prec = 3
     err = None
     datatype = None
     
@@ -155,9 +185,7 @@ class MatrixWithCoords:
         ret.xc_label = self.xc_label
         ret.yc_label = self.yc_label
         shape = (len(ret.yc),len(ret.xc))
-        
-        #print(common_x,common_y,len(ret.xc),len(ret.yc),shape)
-        
+                
         wt = np.zeros(shape)
         ret.d = np.zeros(shape)
         if self.err is not None:
@@ -188,45 +216,6 @@ class MatrixWithCoords:
 
         return ret
     
-    def conv0(self, Nx1, Ny1, xc1, yc1, mask=None, cor_factor=1, datatype=DataType.det):
-        """ re-organize the 2D data based on new coordinates (xc1,yc1) for each pixel
-            returns a new MatrixWithCoords, with the new coordinates specified by Nx1, Ny1
-            Nx1, Ny1 can be either the number of bins or an array that specifies the bin edges
-            datatype is used to describe the type of the 2D data (detector image, qr-qz map, q-phi map)
-        """
-        ret = MatrixWithCoords()
-        ret.datatype = datatype
-
-        # correction factor is applied by dividing its value
-        # should be called like this conv(...., cor_factor=exp.FSA)
-        data = self.d/cor_factor
-        if mask is None:
-            xc1 = xc1.flatten()
-            yc1 = yc1.flatten()
-            data = data.flatten()
-        else:
-            xc1 = xc1[~(mask.map)].flatten()
-            yc1 = yc1[~(mask.map)].flatten()
-            data = data[~(mask.map)].flatten()
-
-        if hasattr(Nx1, '__iter__'): # tuple or list or np array
-            Nx1 = get_bin_edges(Nx1)
-        if hasattr(Ny1, '__iter__'): # tuple or list or np array
-            Ny1 = get_bin_edges(Ny1)
-
-        (v_map, x_edges, y_edges) = np.histogram2d(xc1, yc1,
-                                                bins=(Nx1, Ny1), weights=data)
-        (c_map, x_edges, y_edges) = np.histogram2d(xc1, yc1,
-                                                bins=(Nx1, Ny1), weights=np.ones(len(data)))
-
-        idx = (c_map<=0) # no data
-        c_map[idx] = 1.
-        v_map[idx] = np.nan
-        ret.d = np.fliplr(v_map/c_map).T
-        ret.xc = (x_edges[:-1] + x_edges[1:])/2
-        ret.yc = (y_edges[:-1] + y_edges[1:])/2
-        return ret
-
     def conv(self, Nx1, Ny1, xc1, yc1, mask=None, interpolate=None, cor_factor=1, 
              inc_stat_err=False, datatype=DataType.det, err_thresh=1e-5):
         """ re-organize the 2D data based on new coordinates (xc1,yc1) for each pixel
@@ -253,12 +242,22 @@ class MatrixWithCoords:
 
         if hasattr(Nx1, '__iter__'): # tuple or list or np array
             Nx1 = get_bin_edges(Nx1)
+            xrange = (np.min(Nx1), np.max(Nx1))
+        else:
+            xrange = (np.min(xc1), np.max(xc1))            
         if hasattr(Ny1, '__iter__'): # tuple or list or np array
             Ny1 = get_bin_edges(Ny1)
-
-        (c_map, x_edges, y_edges) = np.histogram2d(xc1, yc1, bins=(Nx1, Ny1), weights=np.ones(len(data)))
-        (xc_map, x_edges, y_edges) = np.histogram2d(xc1, yc1, bins=(Nx1, Ny1), weights=xc1)
-        (yc_map, x_edges, y_edges) = np.histogram2d(xc1, yc1, bins=(Nx1, Ny1), weights=yc1)
+            yrange = (np.min(Ny1), np.max(Ny1))
+        else:
+            yrange = (np.min(yc1), np.max(yc1))
+        xyrange = [xrange, yrange]
+        
+        (c_map, x_edges, y_edges) = histogram2d(xc1, yc1, range=xyrange,
+                                                bins=(Nx1, Ny1), weights=np.ones(len(data)))
+        (xc_map, x_edges, y_edges) = histogram2d(xc1, yc1, range=xyrange,
+                                                 bins=(Nx1, Ny1), weights=xc1)
+        (yc_map, x_edges, y_edges) = histogram2d(xc1, yc1, range=xyrange,
+                                                 bins=(Nx1, Ny1), weights=yc1)
         cidx = (c_map>0)
 
         if inc_stat_err:
@@ -267,10 +266,13 @@ class MatrixWithCoords:
             dw[idx] = 1./data[idx]
             counted = np.zeros(len(data))
             counted[idx] = 1.
-            (v_map, x_edges, y_edges) = np.histogram2d(xc1, yc1, bins=(Nx1, Ny1), weights=data*dw)
-            (w_map, x_edges, y_edges) = np.histogram2d(xc1, yc1, bins=(Nx1, Ny1), weights=dw)
+            (v_map, x_edges, y_edges) = histogram2d(xc1, yc1, range=xyrange,
+                                                    bins=(Nx1, Ny1), weights=data*dw)
+            (w_map, x_edges, y_edges) = histogram2d(xc1, yc1, range=xyrange,
+                                                    bins=(Nx1, Ny1), weights=dw)
             # we are skipping the zero intensity pixels
-            (c1_map, x_edges, y_edges) = np.histogram2d(xc1, yc1, bins=(Nx1, Ny1), weights=counted)   
+            (c1_map, x_edges, y_edges) = histogram2d(xc1, yc1, range=xyrange,
+                                                     bins=(Nx1, Ny1), weights=counted)   
             idx = (w_map>0)
             v_map[idx] /= w_map[idx]
             v_map[~idx] = np.nan
@@ -282,8 +284,10 @@ class MatrixWithCoords:
             v_map[idx] /= scl
             e_map[idx] /= scl
         else:
-            (v_map, x_edges, y_edges) = np.histogram2d(xc1, yc1, bins=(Nx1, Ny1), weights=data)
-            (v2_map, x_edges, y_edges) = np.histogram2d(xc1, yc1, bins=(Nx1, Ny1), weights=data*data)
+            (v_map, x_edges, y_edges) = histogram2d(xc1, yc1, range=xyrange,
+                                                    bins=(Nx1, Ny1), weights=data)
+            (v2_map, x_edges, y_edges) = histogram2d(xc1, yc1, range=xyrange,
+                                                     bins=(Nx1, Ny1), weights=data*data)
             v_map[cidx] /= c_map[cidx]
             v_map[~cidx] = np.nan
             v2_map[cidx] /= c_map[cidx]
@@ -295,7 +299,7 @@ class MatrixWithCoords:
         xc_map[cidx] /= c_map[cidx]
         yc_map[cidx] /= c_map[cidx]
 
-        ret.err = np.fliplr(e_map).T
+        ret.err = e_map.T
 
         # re-evaluate based on the actual coordinates instead of the expected
         # unable to do it in 2D
@@ -311,40 +315,79 @@ class MatrixWithCoords:
                 if (len(v_map[i,:][~idx])>2):
                     v_map[i,:][~idx] = np.interp(ret.yc[~idx], yc_map[i,:][~idx], v_map[i,:][~idx])
 
-        ret.d = np.fliplr(v_map).T
+        ret.d = v_map.T
         idx = (ret.d==0) | (ret.err<err_thresh)
         ret.d[idx] = np.nan
         ret.err[idx] = np.nan
         return ret
 
-    def plot(self, ax=None, logScale=False, **kwargs):
+    def plot(self, ax=None, logScale=False, aspect='auto', colorbar=False, sc_factor=None, clim="auto", **kwargs):
         if ax is None:
             plt.figure()
             ax = plt.gca()
 
-        if logScale:
-            ax.imshow(np.log(self.d), **kwargs)
+        # need to fix the direction of y-axis; in HPLC y is q and increases downward; q-phi map is the opposite 
+        xx = np.tile(self.xc, [len(self.yc),1])
+        if sc_factor=="x":
+            sc_factor = xx
+        elif sc_factor=="x2":
+            sc_factor = xx*xx
+        elif sc_factor=="x0.5":
+            sc_factor = np.power(xx, 0.5)
+        elif sc_factor=="x1.5":
+            sc_factor = np.power(xx, 1.5)
         else:
-            ax.imshow(self.d, **kwargs)
+            sc_factor = 1
+        kwargs.pop('sc_factor', None)
+            
+        if clim=="auto":
+            clim = auto_clim(self.d*sc_factor, logScale)
+        if logScale:
+            im = ax.imshow(np.log(self.d*sc_factor), aspect=aspect, clim=np.log(clim), origin="lower", **kwargs)
+        else:
+            im = ax.imshow(self.d*sc_factor, aspect=aspect, clim=clim, origin="lower", **kwargs)
         ax.set_xlabel('ix')
         ax.set_ylabel('iy')
+        ax.format_coord = self.format_coord
+        
+        if aspect=='auto':
+            axx = ax.twiny()
+            gpindex,gpvalues,gplabels = grid_labels(self.xc)
+            axx.set_xticks(gpindex)
+            axx.set_xticklabels(gplabels)
+            if self.xc_label:
+                axx.set_xlabel(self.xc_label)
 
-        axx = ax.twiny()
-        gpindex,gpvalues,gplabels = grid_labels(self.xc)
-        axx.set_xticks(gpindex)
-        axx.set_xticklabels(gplabels)
-        if self.xc_label:
-            axx.set_xlabel(self.xc_label)
+            axy = ax.twinx()
+            gpindex,gpvalues,gplabels = grid_labels(self.yc)
+            axy.set_yticks(gpindex)
+            axy.set_yticklabels(gplabels)
+            if self.yc_label:
+                axy.set_ylabel(self.yc_label)
 
-        axy = ax.twinx()
-        gpindex,gpvalues,gplabels = grid_labels(self.yc)
-        axy.set_yticks(gpindex)
-        axy.set_yticklabels(gplabels)
-        if self.yc_label:
-            axy.set_ylabel(self.yc_label)
-            
-        axy.format_coord = ax.format_coord #make_format(ax2, ax1)
+            axy.format_coord = ax.format_coord #ax.format_coord #make_format(ax2, ax1)
+        
+        if colorbar:
+            plt.colorbar(im)
+        #plt.connect('button_press_event', self.mouse_press)
+        #plt.connect('button_release_event', self.mouse_release)
 
+    def format_coord(self, x, y):
+        ix = x
+        iy = y  
+        msg = f"ix={ix:.1f}, iy={iy:.1f}; "
+        
+        col = int(x+0.5)
+        row = int(iy+0.5)
+        xc0 = self.xc[col] #np.interp(col, np.arange(len(self.xc)), self.xc)
+        yc0 = self.yc[row] #np.interp(row, np.arange(len(self.yc)), self.yc)
+        msg += f"{self.xc_label}={xc0:.{self.xc_prec}f}, {self.yc_label}={yc0:.{self.yc_prec}f}: "
+        
+        if col>=0 and col<len(self.xc) and row>=0 and row<len(self.yc):
+            val = self.d[row][col]
+            msg += f"{val:.2f}  "
+        return msg
+                
     def roi(self, x1, x2, y1, y2, mask=None):
         """ return a ROI within coordinates of x=x1~x2 and y=y1~y2 
         """
@@ -354,7 +397,7 @@ class MatrixWithCoords:
         xidx = (self.xc>=np.min([x1,x2])) & (self.xc<=np.max([x1,x2]))
         yidx = (self.yc>=np.min([y1,y2])) & (self.yc<=np.max([y1,y2]))
         t1 = np.tile(xidx, [len(yidx),1])
-        t2 = np.tile(np.flipud(yidx), [len(xidx),1]).T
+        t2 = np.tile(yidx, [len(xidx),1]).T
 
         ret.xc = self.xc[xidx]
         ret.yc = self.yc[yidx]
@@ -374,56 +417,35 @@ class MatrixWithCoords:
         """ return the averaged value of the data at coordinates (x0,y0), within a box of (dx, dy) 
         """
         roi = self.roi(x0-0.5*dx, x0+0.5*dx, y0-0.5*dy, y0+0.5*dy, mask=mask)
-        d = roi[~np.isnan(roi)]
+        d = roi.d[~np.isnan(roi.d)]
         return np.sum(d)/len(d)
     
-    def line_cut(self, x0, y0, ang, lT, lN, nT, nN, mask=None):
-        """ take a line cut
-                (x0, y0): center
-                ang: orientation (in degrees) 
-                lT, lN: half length (tangential) and half width (normal) of the cut in pixels 
-                lN, nN: number of returned data points logn the legnth and width of the cut
+    def line_profile(self, direction, xrange=None, yrange=None, plot_data=False, ax=None):
+        """ return the line profile along the specified direction ("x" or "y"), 
+            within the range of coordinate given by crange=[min, max] in the other direction
         """
-        ret = MatrixWithCoords()
-        ret.datatype = self.datatype
-        ang = np.radians(ang)
-        
-        if self.yc[0]>self.yc[-1]:  # raw data, y coordinate lower at the top of the image
-            ang = -ang
-        
-        # xc/yc need to have the same dimension as d
-        (h,w) = self.d.shape
-        xc = np.repeat(self.xc, h).reshape((w, h)).T
-        yc = np.repeat(np.flipud(self.yc), w).reshape((h, w))
-        
-        distT =  (xc.flatten()-x0)*np.cos(ang) + (yc.flatten()-y0)*np.sin(ang) 
-        distN = -(xc.flatten()-x0)*np.sin(ang) + (yc.flatten()-y0)*np.cos(ang) 
-
-        bin_t = np.linspace(-lT, lT, nT)
-        bin_n = np.linspace(-lN, lN, nN)
-
-        # exclude masked pixels or pixels that contain non-numerical values
-        if mask is None:
-            idx = np.isnan(self.d)
+        if xrange is None:
+            xrange = [self.xc[0], self.xc[-1]]
+        if yrange is None:
+            yrange = [self.yc[0], self.yc[-1]]
+        if direction=="x":
+            cc = self.xc[(self.xc>=xrange[0]) & (self.xc<=xrange[1])]
+        elif direction=="y":
+            cc = self.yc[(self.yc>=yrange[0]) & (self.yc<=yrange[1])]
         else:
-            idx = mask.map | np.isnan(self.d)
-        idx = ~idx.flatten()
+            raise exception(f"invalid direction: {direction}")
+            
+        roi = self.roi(xrange[0], xrange[1], yrange[0], yrange[1])
+        dd,ee = roi.flatten(axis=direction)
         
-        (v_map, t_edges, n_edges) = np.histogram2d(distT[idx], distN[idx],
-                                                bins=(bin_t, bin_n), weights=self.d.flatten()[idx])
-        (c_map, t_edges, n_edges) = np.histogram2d(distT[idx], distN[idx],
-                                                bins=(bin_t, bin_n), weights=np.ones(len(distT[idx])))
-                                                 
-        ret.d = np.fliplr(v_map/c_map).T    
-        ret.xc = (t_edges[:-1] + t_edges[1:])/2
-        ret.yc = (n_edges[:-1] + n_edges[1:])/2
-        
-        if self.yc.flatten()[0]>self.yc.flatten()[-1]:  # raw data, y coordinate lower at the top of the image
-            ret.d = np.flipud(ret.d)
-        
-        return ret
+        if plot_data:
+            if ax is None:
+                fig,ax = plt.figure()
+            ax.plot(cc,dd)
+
+        return cc,dd,ee
     
-    def flatten(self, axis='x', method='err_weighted'):
+    def flatten(self, axis='x', method='simple'):
         """ collapse the matrix onto the specified axis and turn it into an array 
         """
         if not axis in ['x', 'y']:
@@ -440,14 +462,13 @@ class MatrixWithCoords:
                 dat.append(self.d[i,:])
                 if self.err is not None:
                     err.append(self.err[i,:])
-        if axis=='y':
+            dd,ee = calc_avg(dat, err, method=method)
+        if axis=='y': 
             for i in range(len(self.xc)):
                 dat.append(self.d[:,i])
                 if self.err is not None:
                     err.append(self.err[:,i])
-        if self.err is None:
-            err = None
-        dd,ee = calc_avg(dat, err, method=method)
+            dd,ee = calc_avg(dat, err, method=method)
         
         return dd,ee
 
@@ -483,6 +504,73 @@ class MatrixWithCoords:
             ret.d = self.d - dbkg.d*scale_factor
             if self.err and dbkg.err:
                 ret.err = self.err + dbkg.err*scale_factor
+        
+    def apply_symmetry(self):
+        """ this only applies if the y coordinate is angle and covers 360deg
+        """
+        t = self.copy()
+        Np = int(len(self.yc)/2)
+        t.d = np.vstack([self.d[Np:,:], self.d[:Np,:]])
+        return self.merge([t])
+
+    def fill_gap(self, method="spline", param=0.05):
+        """ 
+            for now this is interepolating in x coordinate only 
+            
+            interpolate within each row 
+            methods should be "linear" or "spline"
+
+            a better version of this should use 2d interpolation
+            but only fill in the space that is narrow enough in one direction (e.g. <5 missing data points)
+        """
+        h,w = self.d.shape
+        ret = self.copy()
+
+        xx1 = np.arange(w)
+        for k in range(h):
+            yy1 = self.d[k,:]    
+            idx = ~np.isnan(yy1)
+            if len(idx)<=10:  # too few valid data points
+                continue
+            idx1 = np.where(idx)[0]
+            # only need to refill the values that are currently nan
+            if len(idx1)>1:
+                idx2 = np.copy(idx)
+                idx2[:idx1[0]] = True
+                idx2[idx1[-1]:] = True
+                if method=="linear":
+                    ret.d[k,~idx2] = np.interp(xx1[~idx2], xx1[idx], yy1[idx])
+                elif method=="spline":
+                    fs = uspline(xx1[idx], yy1[idx])
+                    fs.set_smoothing_factor(param)
+                    ret.d[k,~idx2] = fs(xx1[~idx2])
+                else:
+                    raise Exception(f"unknown method for intepolation: {method}")
+
+        return ret
+                
+                
+                
+def flip_array(d, flip):
+    if flip == 0:
+        return d
+    df = copy.copy(d)
+    if flip<0:
+        df = np.fliplr(df)
+        flip = -flip
+    df = np.rot90(df, flip)
+        
+    return df
+
+def unflip_array(d, flip):
+    if flip==0:
+        return d
+    df = copy.copy(d)
+    df = np.rot90(df, 4-abs(flip))  # rotate -90
+    if flip<0:
+        df = np.fliplr(df)
+        
+    return df
 
 
 class Data2d:
@@ -490,7 +578,7 @@ class Data2d:
         stores the scattering pattern itself, 
     """
 
-    def __init__(self, img, timestamp=None, uid='', exp=None, label='', dtype=None, flat=None):
+    def __init__(self, img, timestamp=None, uid='', exp=None, ignore_flip=False, label='', dtype=None, flat=None):
         """ read 2D scattering pattern
             img can be either a filename (rely on Fabio to deal with the file format) or a numpy array 
         """
@@ -531,7 +619,7 @@ class Data2d:
         # self.im always stores the original image
         # self.data store the array data after the flip operation
         if exp is not None:
-            self.set_exp_para(exp)
+            self.set_exp_para(exp, ignore_flip)
         else:
             # temporarily set data to img, without a defined exp_para
             self.flip(0)
@@ -560,17 +648,15 @@ class Data2d:
             if flip<0, do a mirror operation first
             the absolute value of flip is the number of 90-deg rotations
         """  
-        self.data.d = self.im.copy()
-        if flip == 0:
-            return
-        if flip<0:
-            self.data.d = np.fliplr(self.data.d)
-            flip = -flip
-        for _ in range(flip):
-            self.data.d = np.rot90(self.data.d)
+        self.data.d = flip_array(self.im, flip)
 
-    def set_exp_para(self, exp):
-        self.flip(exp.flip)
+    def set_exp_para(self, exp, ignore_flip=False):
+        """ ignore_flip is useful if the flipped image is saved
+        """
+        if ignore_flip:
+            self.flip(0)
+        else:
+            self.flip(exp.flip)
         (self.height, self.width) = np.shape(self.data.d)
         self.exp = exp
         if exp.ImageHeight!=self.height or exp.ImageWidth!=self.width:
@@ -662,190 +748,3 @@ class Data2d:
         return Iq,dI
 
     
-class Axes2dPlot:
-    """
-        display MatrixWithCoordinates, image origin at the upper left corner
-        option to provide coordinates translation
-    """    
-    def __init__(self, ax, data, exp=None, datatype=DataType.det):
-        """
-        """
-        self.ax = ax
-        self.cmap = plt.get_cmap()
-        self.scale = 'linear'
-        self.d2 = data
-        self.ptns = []
-        self.xlim = None
-        self.ylim = None
-        self.img = None
-        self.coordinate_translation = None
-        self.exp = exp
-        self.n_step = 121
-        self.datatype = datatype
-        self.capture_mouse()
-
-    def capture_mouse(self):
-        self.ax.figure.canvas.mpl_connect('button_press_event', self.clicked)
-        # self.ax.figure.canvas.mpl_connect('motion_notify_event', self.move_event)
-
-    def right_click(self, event):
-        """ display menu to change image color scale etc.
-        """
-        pass
-
-    def clicked(self, event):
-        if event.inaxes != self.ax:
-            return True
-        x = event.xdatta 
-        y = event.ydata 
-        if self.coordinate_translation=="xy2qrqz":
-            (q, phi, qr, qz) = self.exp.calc_from_XY(np.asarray([x]),np.asarray([y]))
-            msg = "qr=%.4f , qz=%.4f" % (qr[0], qz[0])
-        elif self.coordinate_translation=="xy2qphi":
-            (q, phi, qr, qz) = self.exp.calc_from_XY(np.asarray([x]),np.asarray([y]))
-            msg = "q=%.4f, phi=%.1f" % (q[0],phi[0])
-        else:
-            msg = f"({x:.1f}, {y:.1f})"
-        self.ax.set_title(msg, fontsize="small")
-        print(msg)
-        return True
-
-    # best modify d2.xc/d2.yc instead of using xscale/yscale
-    def plot(self, showMask=None, logScale=False, mask_alpha=0.1,
-             aspect=1., xscale=1., yscale=1.):
-
-        dd = np.asarray(self.d2.d, dtype=np.float)
-
-        immax = np.average(dd) + 5 * np.std(dd)
-        immin = np.average(dd) - 5 * np.std(dd)
-        if immin < 0:
-            immin = 0
-
-        if showMask and self.exp:
-            self.ax.imshow(self.exp.mask.map, cmap="gray")
-        if logScale:
-            self.img = self.ax.imshow(dd, aspect=aspect, alpha=1-mask_alpha,
-                                      cmap=self.cmap, interpolation='nearest', norm=LogNorm(),
-                                      extent=[self.d2.xc[0]*xscale, self.d2.xc[-1]*xscale, 
-                                              self.d2.yc[0]*yscale, self.d2.yc[-1]*yscale])
-        else:
-            self.img = self.ax.imshow(dd, vmax=immax, vmin=immin, 
-                                      aspect=aspect, alpha=1-mask_alpha,
-                                      cmap=self.cmap, interpolation='nearest',
-                                      extent=[self.d2.xc[0]*xscale, self.d2.xc[-1]*xscale, 
-                                              self.d2.yc[0]*yscale, self.d2.yc[-1]*yscale]) 
-        self.xlim = [self.d2.xc[0]*xscale, self.d2.xc[-1]*xscale]
-        self.ylim = [self.d2.yc[0]*yscale, self.d2.yc[-1]*yscale]
-
-    def set_color_scale(self, cmap, gamma=1):
-        """ linear, log/gamma
-        """
-        if not gamma == 1:
-            cmap = cmap_map(lambda x: np.exp(gamma * np.log(x)), cmap)
-        self.cmap = cmap
-        if self.img is not None:
-            self.img.set_cmap(cmap)
-
-    # xvalues and yvalues should be arrays of the same length, positons to be markes
-    # used to mark a pixel position, in all types of 2D maps, or a (qr,qz) or (p,phi) pair on the detector image
-    def mark_points(self, xvalues, yvalues, fmt="r+", datatype=None):
-        if self.d2.datatype==DataType.det and self.exp!=None:
-            (Q, Phi, Qr, Qn) = self.exp.calc_from_XY(np.asarray(xvalues), np.asarray(yvalues))
-            if datatype==DataType.qrqz:
-                (xvalues,yvalues) = (Qr,Qn)
-            elif datatype==DataType.qphi:
-                (xvalues,yvalues) = (Q,Phi)
-        elif datatype==DataType.det and self.exp!=None:
-            if self.d2.datatype==DataType.qrqz:
-                (xvalues,yvalues) = self.exp.calc_from_QrQn(np.asarray(xvalues), np.asarray(yvalues))
-            elif self.d2.datatype==DataType.qphi: # need to convert angular unit
-                (xvalues,yvalues) = self.exp.calc_from_QPhi(np.asarray(xvalues), np.asarray(yvalues))
-        elif self.d2.datatype!=datatype and datatype!=None:
-            raise Exception("imcompatible data types:", datatype, self.d2.datatype)
-            
-        ptn = [[xvalues, yvalues], fmt]    
-        self.ptns.append(ptn)
-        self.draw_dec()
-    
-    # xvalues and yvalues can have different lengths 
-    def mark_coords(self, xvalues, yvalues=None, fmt="r-", datatype=None):
-        if datatype==None or datatype==self.d2.datatype: # straight lines between max/min
-            for xv in xvalues:
-                self.ptns.append([[[xv, xv], self.ylim], fmt])
-            for yv in yvalues:
-                self.ptns.append([[self.xlim, [yv, yv]], fmt])
-        elif self.exp==None:
-            raise Exception("undefined ExpPara for the plot.")
-        elif datatype==DataType.q:
-            phi = np.linspace(0, 360., self.n_step)
-            if self.d2.datatype==DataType.det:
-                for q in xvalues:
-                    (xv, yv) = self.exp.calc_from_QPhi(q*np.ones(self.n_step), phi)
-                    self.ptns.append([[xv, yv], fmt])
-            elif self.d2.datatype==DataType.qrqz:
-                for q in xvalues:
-                    self.ptns.append([[q*np.cos(np.radians(phi)), q*np.sin(np.radians(phi))], fmt])
-            elif self.d2.datatype==DataType.qphi:
-                for q in xvalues:
-                    self.ptns.append([[[q, q], self.ylim], fmt])
-        elif self.d2.datatype==DataType.det and datatype==DataType.qrqz:
-            qr_grid = np.linspace(self.exp.Qr.min(), self.exp.Qr.max(), self.n_step)
-            qz_grid = np.linspace(self.exp.Qn.min(), self.exp.Qn.max(), self.n_step)
-            for qr in xvalues:
-                (xv,yv) = self.exp.calc_from_QrQn(qr*np.ones(self.n_step), qz_grid)
-                self.ptns.append([[xv, yv], fmt])
-            for qz in yvalues:
-                (xv,yv) = self.exp.calc_from_QrQn(qr_grid, qz*np.ones(self.n_step))
-                self.ptns.append([[xv, yv], fmt])
-        elif self.d2.datatype==DataType.det and datatype==DataType.qphi:
-            q_grid = np.linspace(self.exp.Q.min(), self.exp.Q.max(), self.n_step)
-            phi_grid = np.linspace(self.exp.Phi.min(), self.exp.Phi.max(), self.n_step)
-            for q in xvalues:
-                (xv,yv) = self.exp.calc_from_QPhi(q*np.ones(self.n_step), phi_grid)
-                self.ptns.append([[xv, yv], fmt])            
-            for phi in yvalues:
-                (xv,yv) = self.exp.calc_from_QPhi(q_grid, phi*np.ones(self.n_step))
-                self.ptns.append([[xv, yv], fmt])
-        else:
-            raise Exception("imcompatible data types:", datatype, self.d2.datatype)
-            
-        self.draw_dec()            
-    
-    def mark_standard(self, std, sym="k:"):
-        """ std should be one of these: 
-            AgBH: mutiples of 0.1076, then 1.37
-            sucrose: 0.5933, 0.8289, 0.9054, 0.9336, 1.1000
-            CeO2: 2.0116, 1.8985, 3.2838, 3.8504
-            LaB6: 1.5115, 2.1376, 2.6180, 3.0230, 3.3799, 3.7025
-        """
-        if std=="AgBH":
-            q_std = np.hstack((0.1076*np.arange(1,10), [1.37]))
-        elif std=="sucrose":
-            q_std = [0.5933, 0.8289, 0.9054, 0.9336, 1.1000]
-        elif std=="CeO2": # http://nvlpubs.nist.gov/nistpubs/Legacy/MONO/nbsmonograph25-20.pdf
-            q_std = [ 2.0116, 2.3219, 3.2838, 3.8504 ]
-        elif std=="LaB6": # NIST SRM 660c
-            q_std = [ 1.5115, 2.1376, 2.6180, 3.0230, 3.3799, 3.7025 ]
-        elif isinstance(std, list):
-            q_std = np.asarray(std, dtype=np.float)
-        else: # unknown standard
-            return 
-        self.mark_coords(q_std, [], sym, DataType.q)
-        if self.exp and self.datatype==DataType.det:
-            self.mark_points([self.exp.bm_ctr_x], [self.exp.bm_ctr_y], datatype=DataType.det)
-    
-    # simply plot the x,y coordinates
-    def mark_line(self, xv, yv, fmt="r-"):
-        self.ptns.append([[xv, yv], fmt])
-        self.draw_dec()
-    
-    # ptn = [data, fmt]
-    # 
-    def draw_dec(self):
-        for ptn in self.ptns:
-            ([px, py], fmt) = ptn
-            self.ax.plot(px, py, fmt)
-        self.ax.set_xlim(self.xlim)
-        self.ax.set_ylim(self.ylim)
-        
-
