@@ -7,12 +7,14 @@ import time,datetime
 import os,copy,subprocess,re
 import json,pickle,fabio
 import multiprocessing as mp
+import numbers
 
 from py4xs.slnxs import Data1d,average,filter_by_similarity,trans_mode,estimate_scaling_factor
 from py4xs.utils import common_name,max_len,Schilling_p_value
 from py4xs.detector_config import create_det_from_attrs
-from py4xs.local import det_names,det_model,beamline_name,incident_monitor_field,transmitted_monitor_field
-from py4xs.data2d import Data2d,Axes2dPlot,MatrixWithCoords,DataType
+from py4xs.local import det_names,det_model,beamline_name,incident_monitor,transmitted_monitor
+from py4xs.data2d import Data2d,MatrixWithCoords,DataType
+from py4xs.plot import show_data,show_data_qxy,show_data_qphi
 from py4xs.utils import run
 from itertools import combinations
 
@@ -21,6 +23,11 @@ from scipy.ndimage.filters import gaussian_filter
 from scipy.interpolate import UnivariateSpline as uspline
 from scipy.integrate import simpson
 
+from dask.distributed import Client,LocalCluster
+import dask.array as da
+
+
+
 def lsh5(hd, prefix='', top_only=False, silent=False, print_attrs=True):
     """ list the content of a HDF5 file
         
@@ -57,25 +64,77 @@ def create_linked_files(fn, fnlist):
         fs.close()
     ff.close()
 
-def integrate_mon(em, ts, ts0, exp, extend_mon_stream):
+def synch_mon(ts0, em0, ts, em, dt="auto", Ns=8, plot=False):
+    """ revise the timestamp ts (np array) to match the monitor em to em0
+        look for the most significant change, i.e. when the shutter opens 
+        
+        if dt is a numberical value, simply add that value to ts
+        if dt is "auto", in principle both em's should see the shutter openning when the scan starts
+        
+        Ns specifies how much smoothing will be applied to the em data before synching
+    """
+    if isinstance(dt, numbers.Number):
+        ts += ts0[0]-ts[0]+dt
+        return dt
+    if dt!="auto": 
+        return 0   # don't know what to do
+    
+    ts += ts0[0]-ts[0]
+    df = np.diff(em)
+    df0 = np.diff(em0)
+    df /= np.max(df)
+    df0 /= np.max(df0)
+
+    tt = np.linspace(-Ns, Ns, 2*Ns+1)
+    smth = np.exp(-tt**2/Ns)
+    df = np.convolve(df, smth, "same")
+    df0 = np.convolve(df0, smth, "same")
+    idx = df>0.4*np.max(df)
+    idx0 = df0>0.4*np.max(df0)
+    ats = np.average(ts[1:][idx], weights=df[idx])
+    ats0 = np.average(ts0[1:][idx0], weights=df0[idx0])
+    
+    if plot:
+        plt.figure()
+        plt.plot(ts[1:][idx], df[idx])
+        plt.plot(ts0[1:][idx0], df0[idx0])
+        plt.bar([ats], [2], width=0.01)
+        plt.bar([ats0], [2], width=0.01)
+
+    dt = ats0-ats
+    ts += dt
+    return dt  
+    
+def integrate_mon(em, ts, ts0, exp, extend_mon_stream, ts_pos_in_frame="start", debug=False):
     """ integrate monitor counts
         monitor counts are given by em with timestamps ts
         ts0 is the timestamps on the exposures, with duration of exp
-        
+        if extend_mon_stream, repeat the ts data at the beginning and the end to enclose the range of ts0 
         assume ts and ts0 are 1d arrays
+        
+        ts_pos_in_frame: "start" or "end"
+            whether the timestamp correspond to the start of the exposure or the end of the exposure
     """
-    if extend_mon_stream and ts[-1]<ts0[-1]+exp*1.5:
-        em = np.append(em, em[-1])
-        ts = np.append(ts, ts0[-1]+exp*1.5)
-    ffe = interp1d(ts, em)
+    if debug:
+        print(f"integrating monitor counts, time stamp offset mon-tr={ts[0]-ts0[0]:.1f}")
+    if extend_mon_stream: 
+        em = np.concatenate(([em[0]], em, [em[-1]]))
+        ts = np.concatenate(([np.min([ts0[0], ts[0]])-exp*1.5], ts, [np.max([ts0[-1], ts[-1]])+exp*1.5]))
+    ffe = interp1d(ts, em) 
     em0 = []
+    
+    ti0 = {"start": 0, "end": -exp}
+    ti1 = {"start": exp, "end": 0}
+    
     for t in ts0:
-        tt = np.concatenate(([t], ts[(ts>t) & (ts<t+exp)], [t+exp]))
+        tt = np.concatenate(([t+ti0[ts_pos_in_frame]], 
+                              ts[(ts>t+ti0[ts_pos_in_frame]) & (ts<t+ti1[ts_pos_in_frame])], 
+                              [t+ti1[ts_pos_in_frame]])) 
         ee = ffe(tt)
         em0.append(simpson(ee, tt))
     return np.asarray(em0)/exp
 
-def get_monitor_counts(grp, fieldName):
+def get_monitor_counts(grp, monitorName):
     """ look under a data group (grp) that belong to a specific sample, find the stream that contains fieldName
         caluclate the monitor counts based on the given timestamps (ts) and exposure time
     """
@@ -83,23 +142,33 @@ def get_monitor_counts(grp, fieldName):
     for stream in list(grp):
         if not 'data' in list(grp[stream]):
             continue
-        if fieldName in list(grp[stream]["data"]):
-            strn = stream
-            break
+        for fieldName in list(grp[stream]["data"]):
+            if monitorName in fieldName:
+                strn = stream
+                data = grp[strn]["data"][fieldName][...]
+                ts = grp[strn]["timestamps"][fieldName][...]
+                break
     if strn is None:
-        raise Exeption(f"could not find the stream that contains {fieldName}.")
+        raise Exception(f"could not find the stream that contains {monitorName}.")
+
+    # in the case of timeseries data, the time stamps need to be lengthened
+    n = int(len(data)/len(ts))
+    if n>1:
+        # sometimes the first two timestamps are the same; based on the data, this seems to be 
+        # an error, and the first timestamp should to be forward adjusted 
+        if ts[0]==ts[1]:
+            ts[0] -= (ts[2]-ts[1])
+            #data = np.reshape(data, [-1, len(ts)])
+            #ts = ts[1:]
+            #data = data[1:, :]
+            #data = data.flatten()
+        dt = np.average(np.diff(ts))  # the intervals appear to be constant in the data
+        # assuming that the time stamp correspond to the time of read
+        ts = np.linspace(ts[0]-dt, ts[-1], len(data))
     
-    data = grp[strn]["data"][fieldName][...]
-    ts = grp[strn]["timestamps"][fieldName][...]
-
     return strn,ts,data
 
 
-
-
-
-
-
 def pack_d1(data, ret_trans=True):
     """ utility function to creat a list of [intensity, error] from a Data1d object 
         or from a list of Data1s objects
@@ -190,90 +259,7 @@ def merge_d1s(d1s, detectors, save_merged=False, debug=False):
         
     return s0
 
-
-# copied from pipeline-test: merge, fix_angular_range, interp_d2
-def merge(ds):
-    """ merge a list of MatrixWithCoord together
-        the datatype should be DataType.qphi
-    """
-    if len(ds)==1:
-        return ds[0].copy()
-    
-    wt = np.zeros(ds[0].shape)
-    avg = np.zeros(ds[0].shape)
-    idx = None
-    for d in ds:
-        if d.shape!=avg.shape:
-            raise Exception("merge: the two data sets must have the same shape: ", d.shape, avg.shape)
-        idx = ~np.isnan(d)
-        avg[idx] += d[idx]
-        wt[idx] += 1
-
-    idx = (wt>0)
-    avg[idx] /= wt[idx]
-    avg[~idx] = np.nan
-    
-    return avg
-    
-def fix_angular_range(da):
-    """ da should be a numpy array
-        return modified angular range between -180 and 180
-        assume that the angular value is not too far off to begin with
-    """
-    da1 = np.copy(da)
-    da1[da1>180] -= 360    # worse case some angles may go up to 360+delta 
-    da1[da1<-180] += 360   # this shouldn't happen
-    return da1
-
-def interp_d2(d2, method="spline", param=0.05):
-    """ d2 is a 2d array
-        interpolate within each row 
-        methods should be "linear" or "spline"
-        
-        a better version of this should use 2d interpolation
-        but only fill in the space that is narrow enough in one direction (e.g. <5 missing data points)
-        
-    """
-    h,w = d2.shape
-    
-    xx1 = np.arange(w)
-    for k in range(h):
-        yy1 = d2[k,:]    
-        idx = ~np.isnan(yy1)
-        if len(idx)<=10:  # too few valid data points
-            continue
-        idx1 = np.where(idx)[0]
-        # only need to refill the values that are currently nan
-        idx2 = np.copy(idx)
-        idx2[:idx1[0]] = True
-        idx2[idx1[-1]:] = True
-        if method=="linear":
-            d2[k,~idx2] = np.interp(xx1[~idx2], xx1[idx], yy1[idx])
-        elif method=="spline":
-            fs = uspline(xx1[idx], yy1[idx])
-            fs.set_smoothing_factor(param)
-            d2[k,~idx2] = fs(xx1[~idx2])
-        else:
-            raise Exception(f"unknown method for intepolation: {method}")
-
-
-def proc_2d(queue, images, sn, nframes, detectors, qphi_range, debug, starting_frame_no=0):
-    """ convert 2D data to q-phi map
-        may want to do this separately for SAXS and WAXS; how to specify?
-    """
-    pass
-
-def proc_make_thumnails(queue, images, sn, nframes, detectors, qphi_range, debug, starting_frame_no=0):
-    """ make thumbnails, specify the detector, output dataset name, color scale, etc.
-    """
-    pass
-
-def proc_line_profile(queue, images, sn, nframes, detectors, qphi_range, debug, starting_frame_no=0):
-    """ put the results in a dataset, with attributes describing where the results come from?
-    """
-    pass
-
-def proc_d1merge(args):
+def proc_merge1d(args):
     """ utility function to perfrom azimuthal average and merge detectors
     """
     images,sn,nframes,starting_frame_no,debug,detectors,qgrid,reft,save_1d,save_merged,dtype = args
@@ -307,40 +293,6 @@ def proc_d1merge(args):
 
     return [sn, starting_frame_no, ret]
         
-def proc_sample(queue, images, sn, nframes, detectors, qgrid, reft, save_1d, save_merged, debug,
-               starting_frame_no=0, transMode=None, monitor_counts=None):
-    """ utility function to perfrom azimuthal average and merge detectors
-    """
-    ret = {'merged': []}
-    sc = {}
-    
-    for det in detectors:
-        ret[det.extension] = []
-        if det.fix_scale is not None:
-            sc[det.extension] = 1./det.fix_scale
-
-    if debug is True:
-        print("processing started: sample = %s, starting frame = #%d" % (sn, starting_frame_no))
-    for i in range(nframes):
-        for det in detectors:
-            dt = Data1d()
-            label = "%s_f%05d%s" % (sn, i+starting_frame_no, det.extension)
-            dt.load_from_2D(images[det.extension][i+starting_frame_no], 
-                            det.exp_para, qgrid, det.pre_process, det.exp_para.mask,
-                            save_ave=False, debug=debug, label=label)
-            dt.scale(sc[det.extension])
-            ret[det.extension].append(dt)
-    
-        dm = merge_d1s([ret[det.extension][i] for det in detectors], detectors, save_merged, debug)
-        ret['merged'].append(dm)
-            
-    if debug is True:
-        print("processing completed: ", sn, starting_frame_no)
-    if queue is None: # single-thread
-        return ([sn,starting_frame_no,ret])
-    else: # multi-processing    
-        queue.put([sn,starting_frame_no,ret])
-
 class h5exp():
     """ empty h5 file for exchanging exp_setup/qgrid
     """
@@ -352,6 +304,7 @@ class h5exp():
             self.detectors, self.qgrid = exp_setup
             self.save_detectors()
         
+        
     def save_detectors(self):
         self.fh5 = h5py.File(self.fn, "w")   # new file
         dets_attr = [det.pack_dict() for det in self.detectors]
@@ -497,8 +450,13 @@ class h5xs():
         Transmitted beam intensity can be set either from the water peak (sol), or from intensity monitor.
         Data processing can be done either in series, or in parallel. Serial processing can be forced.
         
+        keep a h5 file handle for reading, self.fh5
+        re-open the file for writing only when necessary
+            save_attributes??
+            save_detectors(), save_d1s()
+        
     """    
-    def __init__(self, fn, exp_setup=None, transField=transmitted_monitor_field, save_d1=True, 
+    def __init__(self, fn, exp_setup=None, transField=transmitted_monitor, save_d1=True, 
                  have_raw_data=True, read_only=False):
         """ exp_setup: [detectors, qgrid]
             transField: the intensity monitor field packed by suitcase from databroker
@@ -512,14 +470,13 @@ class h5xs():
         self.attrs = {}
         # name of the dataset that contains transmitted beam intensity, e.g. em2_current1_mean_value
         self.transField = ''  
-        self.transStream = {}  
+        self.transStream = {} 
+        self.read_only = read_only
+        self.writable = False   # this is the current state of th eh5 file
 
         self.fn = fn
-        if read_only:
-            self.fh5 = h5py.File(self.fn, "r")   # file must exist
-        else:
-            self.fh5 = h5py.File(self.fn, "r+")   # file must exist
-        if exp_setup==None:     # assume the h5 file will provide the detector config
+        self.fh5 = h5py.File(fn, "r", swmr=True)   # file must exist; reopen for writing only when necessary
+        if exp_setup is None:     # assume the h5 file will provide the detector config
             self.qgrid = self.read_detectors()
         else:
             self.detectors, self.qgrid = exp_setup
@@ -580,14 +537,39 @@ class h5xs():
                 self.transMode = trans_mode.external
 
             if not read_only:
+                self.enable_write(True)
                 self.fh5.attrs['trans'] = ','.join([str(self.transMode.value), self.transField])  #elf.transStream])
-                self.fh5.flush()
-            
+                self.enable_write(False)
+                
+    def enable_write(self, writable):
+        if self.read_only:
+            if not writable:
+                return
+            raise Exception(f"attempting to enable writing to a read-only h5 file ...")
+        if self.writable == writable:
+            return
+        self.fh5.close()
+        if writable:
+            self.fh5 = h5py.File(self.fn, "r+")
+            #self.fh5.swmr_mode = True
+        else:
+            self.fh5 = h5py.File(self.fn, "r", swmr=True)
+        self.writable = writable
+    
+    def det(self, ext):
+        for d in self.detectors:
+            if d.extension==ext:
+                return d
+        return None
+    
     def save_detectors(self):
+        if self.read_only:
+            return
+        self.enable_write(True)
         dets_attr = [det.pack_dict() for det in self.detectors]
         self.fh5.attrs['detectors'] = json.dumps(dets_attr)
         self.fh5.attrs['qgrid'] = list(self.qgrid)
-        self.fh5.flush()
+        self.enable_write(False)
     
     def read_detectors(self):
         dets_attr = self.fh5.attrs['detectors']
@@ -595,11 +577,13 @@ class h5xs():
         self.detectors = [create_det_from_attrs(attrs) for attrs in json.loads(dets_attr)]  
         return np.asarray(qgrid)
 
-    def dset(self, dsname, data_type="data", sn=None):
+    def dset(self, dsname, data_type="data", sn=None, get_path=False):
         assert (data_type in ["data", "timestamps"])
         if sn is None:
             sn = self.samples[0]
         strm = find_field(self.fh5, dsname, sn)
+        if get_path:
+            return f"{sn}/{strm}/{data_type}/{dsname}"
         return self.fh5[f"{sn}/{strm}/{data_type}/{dsname}"]
     
     def md_dict(self, sn, md_keys=[]):
@@ -736,32 +720,45 @@ class h5xs():
         frn = self.verify_frn(sn, frn, flatten=True)
         return self.d1s[sn][group][frn]    
             
-    def get_d2(self, sn=None, det_ext=None, frn=None, dtype=None):
+    def get_d2(self, sn=None, det_ext=None, frn=None, dtype=None, detectors=None, client=None):
         if sn is None:
             sn = self.samples[0]
         d2s = {}
-        for det in self.detectors:
+        if detectors is None:
+            detectors = self.detectors
+            
+        if frn=="average" and client is None:
+            raise Exception("need a dask client for calculating the average.")
+            
+        for det in detectors:
             try:
                 dset = self.fh5[f"{sn}/primary/data/{self.det_name[det.extension]}"]
             except:
                 continue
-            frn = self.verify_frn(sn, frn)
-            d2 = Data2d(dset[tuple(frn)], exp=det.exp_para, dtype=dtype)
+            if frn=="average":
+                imgs = da.array(dset)
+                fut = da.average(imgs, axis=tuple(range(len(dset.shape)-2)))
+                davg = fut.compute(client=client)
+                d2 = Data2d(davg, exp=det.exp_para, dtype=dtype)
+            else:
+                frn = self.verify_frn(sn, frn)
+                d2 = Data2d(dset[tuple(frn)], exp=det.exp_para, dtype=dtype)
             d2.md["frame #"] = frn 
             d2s[det.extension] = d2
+
         if not det_ext:
             return d2s
         else:
             return d2s[det_ext]
                 
     def check_bm_center(self, sn=None, det_ext='_SAXS', frn=0, 
-                        qs=0.005, qe=0.05, qn=100, Ns=9):
+                        qs=0.005, qe=0.05, qn=100, Ns=9, **kwargs):
         """ this function compares the beam intensity on both sides of the beam center,
             and advise if the beam center as defined in the detector configuration is incorrect
             dividing data into 4*Ns slices, show data in horizontal and vertical cuts
         """
         i = 0
-        d2 = self.get_d2(sn=sn, det_ext=det_ext, frn=frn)
+        d2 = self.get_d2(sn=sn, det_ext=det_ext, frn=frn, **kwargs)
         qg = np.linspace(qs, qe, qn)
         d2.conv_Iqphi(Nq=qg, Nphi=Ns*4, mask=d2.exp.mask)
 
@@ -838,190 +835,71 @@ class h5xs():
         print(f"1 data point = {dx:.2f} pixels")
         plt.show()
         
-    def show_data(self, sn=None, det_ext='_SAXS', frn=None, ax=None,
-                  logScale=True, showMask=False, mask_alpha=0.1, 
-                  clim=(0.1,14000), showRef=["AgBH", "r:"], cmap=None, dtype=None):
-        """ display frame #frn of the data under det for sample sn
+    def show_data(self, sn=None, det_ext=None, frn=None, ax=None, fig=None, aspect=1,
+                  logScale=True, showMask=False, mask_alpha=0.1, clim='auto', ch_thresh=15,
+                  showRef=["AgBH", "r:"], cmap=None, detectors=None, dtype=None, **kwargs):
+        """ display frame #frn of the data for sample sn and frame number frn
+            if not specified, take the first sample and/or first frame
+
+            if det_ext is None, show data from all detectors
+            otherwise show the data that correspond to the specified detector extension, e.g. "_SAXS"
+
+            when clim is 'auto', check the distribution/histogram of intensity, set limits 
+                to include only bins with number of samples greater than ch_thresh
         """
-        d2 = self.get_d2(sn=sn, det_ext=det_ext, frn=frn, dtype=dtype)
-        if ax is None:
-            plt.figure()
-            ax = plt.gca()
-        pax = Axes2dPlot(ax, d2.data, exp=d2.exp)
-        if cmap is not None:
-            pax.set_color_scale(plt.get_cmap(cmap)) 
-        pax.plot(logScale=logScale, showMask=showMask, mask_alpha=mask_alpha)
-        pax.img.set_clim(*clim)
-        pax.coordinate_translation="xy2qphi"
-        if showRef:
-            pax.mark_standard(*showRef)
-        ax.set_title(f"frame #{d2.md['frame #']}")
-        pax.capture_mouse()
-        plt.show() 
+        if detectors is None:
+            detectors = self.detectors
+        d2s = self.get_d2(sn=sn, det_ext=det_ext, frn=frn, dtype=dtype, detectors=detectors, **kwargs)
+        show_data(d2s, ax=ax, fig=fig, aspect=aspect,
+                  logScale=logScale, showMask=showMask, mask_alpha=mask_alpha, 
+                  clim=clim, showRef=showRef, cmap=cmap, dtype=dtype, **kwargs)
+        return d2s
     
-    def show_data_qxy(self, sn=None, frn=None, ax=None, dq=0.006,
-                      fig_type="qxy", apply_sym=False, fix_gap=False, bkg=None,
-                      logScale=True, useMask=True, clim=(0.1,14000), showRef=True, 
-                      aspect='auto', cmap=None, dtype=None, colorbar=False):
+    def show_data_qxy(self, sn=None, frn=None, ax=None, dq=0.01, bkg=None,
+                      logScale=True, useMask=True, clim=(0.1,14000), 
+                      aspect=1, cmap=None, detectors=None, dtype=None, colorbar=False, **kwargs):
         """ display frame #frn of the data under det for sample sn
             det is a list of detectors, or a string, data file extension
-            fig_type should be either "qxy" or "qphi"
         """
-        d2s = self.get_d2(sn=sn, frn=frn, dtype=dtype)
-        if ax is None:
-            plt.figure()
-            ax = plt.gca()
-
-        xqmax = np.max([d.exp_para.xQ.max() for d in self.detectors])
-        xqmin = np.min([d.exp_para.xQ.min() for d in self.detectors])
-        yqmax = np.max([d.exp_para.yQ.max() for d in self.detectors])
-        yqmin = np.min([d.exp_para.yQ.min() for d in self.detectors])
-
-        xqmax = np.floor(xqmax/dq)*dq
-        xqmin = np.ceil(xqmin/dq)*dq
-        yqmax = np.floor(yqmax/dq)*dq
-        yqmin = np.ceil(yqmin/dq)*dq
-
-        xqgrid = np.arange(start=xqmin, stop=xqmax+dq, step=dq)
-        yqgrid = np.arange(start=yqmin, stop=yqmax+dq, step=dq)        
-
-        xyqmaps = []
-        for det in self.detectors:
-            dn = det.extension
-            if useMask:
-                mask = d2s[dn].exp.mask
-            else:
-                mask = None
-            
-            cor_factor = d2s[dn].exp.FSA*d2s[dn].exp.FPol
-            if det.flat is not None:
-                cor_factor = det.flat*cor_factor
-            dm = d2s[dn].data.conv(xqgrid, yqgrid, d2s[dn].exp.xQ, d2s[dn].exp.yQ, 
-                                   mask=mask, cor_factor=cor_factor)
-            
-            if bkg is not None:
-                if dn in bkg.keys():
-                    dbkg = Data2d(bkg[dn], exp=det.exp_para)
-                    dm_b = dbkg.data.conv(xqgrid, yqgrid, d2s[dn].exp.xQ, d2s[dn].exp.yQ, 
-                                          mask=mask, cor_factor=cor_factor)
-                    dm.d -= dm_b.d    
-            
-            dm.d *= (d2s[dn].exp.Dd/d2s["_SAXS"].exp.Dd)**2
-            xyqmaps.append(dm.d)
-        
-        xyqmap = merge(xyqmaps)
-        if logScale:
-            im = ax.imshow(np.log(xyqmap), extent=(xqmin, xqmax, yqmin, yqmax), aspect=aspect, cmap=cmap)
-            im.set_clim(np.log(clim))
-        else:
-            im = ax.imshow(xyqmap, extent=(xqmin, xqmax, yqmin, yqmax), aspect=aspect, cmap=cmap)
-            im.set_clim(clim)
-
-        ax.set_title(f"frame #{d2s[list(d2s.keys())[0]].md['frame #']}")
-        if colorbar:
-            plt.colorbar(im)
-
+        d2s = self.get_d2(sn=sn, frn=frn, dtype=dtype, **kwargs)
+        if detectors is None:
+            detectors = self.detectors
+        return show_data_qxy(d2s, detectors, ax=ax, dq=dq, bkg=bkg,
+                             logScale=logScale, useMask=useMask, clim=clim, 
+                             aspect=aspect, cmap=cmap, colorbar=colorbar, **kwargs)
 
     def show_data_qphi(self, sn=None, frn=None, ax=None, Nq=200, Nphi=60,
-                       apply_symmetry=False, fill_gap=False, interp_method='linear',
+                       apply_symmetry=False, fill_gap=False, sc_factor=None,
                        logScale=True, useMask=True, clim=(0.1,14000), showRef=True, bkg=None,
-                       aspect="auto", cmap=None, dtype=None, colorbar=False):
-        d2s = self.get_d2(sn=sn, frn=frn, dtype=dtype)
-        if ax is None:
-            plt.figure()
-            ax = plt.gca()
-
-        qmax = np.max([d.exp_para.Q.max() for d in self.detectors]) 
-        qmin = np.min([d.exp_para.Q.min() for d in self.detectors]) 
-        # keep 2 significant digits only for the step_size
-        dq = (qmax-qmin)/Nq
-        n = int(np.floor(np.log10(dq)))
-        sc = np.power(10., n)
-        dq = np.around(dq/sc, 1)*sc
-
-        qmax = dq*np.ceil(qmax/dq)
-        qmin = dq*np.floor(qmin/dq)
-        Nq = int((qmax-qmin)/dq+1)
-
-        q_grid = np.linspace(qmin, qmax, Nq) 
-
-        Nphi = 2*int(Nphi/2)+1
-        phi_grid = np.linspace(-180., 180, Nphi)
-
-        dms = []
-        for det in self.detectors:
-            dn = det.extension
-            if not dn in d2s.keys():
-                continue
-            if useMask:
-                mask = d2s[dn].exp.mask
-            else:
-                mask = None
-            
-            cor_factor = d2s[dn].exp.FSA*d2s[dn].exp.FPol
-            if det.flat is not None:
-                cor_factor = det.flat*cor_factor
-                
-            # since the q/phi grids are specified, use the MatrixWithCoord.conv() function
-            # the grid specifies edges of the bin for histogramming
-            # the phi value in exp_para may not always be in the range of (-180, 180)
-            dm = d2s[dn].data.conv(q_grid, phi_grid, d2s[dn].exp.Q, 
-                                   fix_angular_range(d2s[dn].exp.Phi),
-                                   cor_factor=cor_factor,
-                                   mask=mask, datatype=DataType.qphi)
-            
-            if bkg is not None:
-                if dn in bkg.keys():
-                    dbkg = Data2d(bkg[dn], exp=det.exp_para)
-                    dm_b = dbkg.data.conv(q_grid, phi_grid, d2s[dn].exp.Q,
-                                          fix_angular_range(d2s[dn].exp.Phi),
-                                          cor_factor=cor_factor,
-                                          mask=mask, datatype=DataType.qphi)
-                    dm.d -= dm_b.d    
-
-            d1 = dm.d/det.fix_scale
-            if apply_symmetry:
-                Np = int(Nphi/2)
-                d2 = np.vstack([d1[Np:,:], d1[:Np,:]])
-                d1 = merge([d1,d2])
-
-            # not attempt to extrapolate into non-coveraged area in reciprocal space
-            # doing interpolation here since the non-coverage area is well defined for 
-            # a single detector, but may not be in merged data 
-            if fill_gap:
-                interp_d2(d1, method=interp_method)
-
-            dms.append(d1)
-        
-        qphimap = merge(dms)
-        if logScale:
-            im = ax.imshow(np.log(qphimap), extent=(qmin, qmax, -180, 180), aspect=aspect, cmap=cmap)
-            im.set_clim(np.log(clim))
-        else:
-            im = ax.imshow(qphiqmap, extent=(qmin, qmax, -180, 180), aspect=aspect, cmap=cmap)
-            im.set_clim(clim)
-
-        ax.set_title(f"frame #{d2s[list(d2s.keys())[0]].md['frame #']}")
-        if colorbar:
-            plt.colorbar(im)
+                       aspect="auto", cmap=None, detectors=None, dtype=None, colorbar=False, **kwargs):
+        d2s = self.get_d2(sn=sn, frn=frn, dtype=dtype, **kwargs)
+        if detectors is None:
+            detectors = self.detectors
+        return show_data_qphi(d2s, detectors, ax=ax, Nq=200, Nphi=60,
+                              apply_symmetry=apply_symmetry, fill_gap=fill_gap, 
+                              sc_factor=sc_factor, bkg=bkg,
+                              logScale=logScale, useMask=useMask, clim=clim, 
+                              aspect=aspect, cmap=cmap, colorbar=colorbar, **kwargs)
 
+    
     def get_mon(self, sn=None, trigger=None, gf_sigma=2, exp=1, 
-                force_synch=-0.25, force_synch_trig=0, extend_mon_stream=True,
-                debug=False, plot_trigger=False, **kwargs): 
+                force_synch='auto', force_synch_trig=0, extend_mon_stream=True,
+                plot_trigger=False, **kwargs): 
         """ calculate the monitor counts for each data point
             1. if the monitors are read together with the detectors 
             2. if the monitors are used asynchronously, monitor values would need to be integated 
-               based on timestampls; a trigger must be provided 
+               based on timestamps; a trigger must be provided 
                 2a: in fly scans, this should be a motor name
                 2b: for solution scattering, use "sol" as trigger
 
             the timestamps on the trigger and em1/em2 may not be synced, dt0 provides a correction 
-            to the trigger timestamp
+            to the trigger timestamp; 
             
             if plot_trigger=True and a single sample is named as sn, generate a plot for verification
 
             timestamps on em1 appear to be way off, ntpd not running
-            if force_synch is non-zero, use first em2 timestamp + force_synch as start of em1 
+            if force_synch is non-zero, use first em2 timestamp + force_synch as start of em1
+            in principle can be synched by the shutter opening seen by the monitors when the scan starts
             
             in some cases, the monitor stream appears to end before data collection is complete
             if extend_mon_stream=True, repeat the last mon data point until the timestamp covers
@@ -1042,10 +920,10 @@ class h5xs():
                     exp=md['exposure_time']
 
             # expect trans and incid monitor data to be in the same stream
-            strn,ts2,trans_data = get_monitor_counts(self.fh5[sn], transmitted_monitor_field)
-            strn,ts1,incid_data = get_monitor_counts(self.fh5[sn], incident_monitor_field)
-            if force_synch!=0: # timestamps between em1/em2 
-                ts1 = ts1-ts1[0]+ts2[0]+force_synch
+            strn,ts2,trans_data = get_monitor_counts(self.fh5[sn], transmitted_monitor)
+            strn,ts1,incid_data = get_monitor_counts(self.fh5[sn], incident_monitor)
+            # synch em1 time stamps to em2, LiX-specific 
+            synch_mon(ts2, trans_data, ts1, incid_data, dt=force_synch)
                 
             if strn=="primary":
                 print("monitors are used as detectors.")
@@ -1077,15 +955,17 @@ class h5xs():
                 incid_data0 = incid_data
             else:
                 try:
-                    trans_data0 = integrate_mon(trans_data, ts2, ts0+force_synch_trig, exp, extend_mon_stream)
-                    incid_data0 = integrate_mon(incid_data, ts1, ts0+force_synch_trig, exp, extend_mon_stream)                
+                    trans_data0 = integrate_mon(trans_data, ts2, ts0+force_synch_trig, exp, extend_mon_stream, **kwargs)
+                    incid_data0 = integrate_mon(incid_data, ts1, ts0+force_synch_trig, exp, extend_mon_stream, **kwargs)   
                 except:
                     t0 = np.min(ts2)
                     print(f"time series likely misaligned:")
                     print(f"trans mon: {np.min(ts2)-t0} ~ {np.max(ts2)-t0}")
                     print(f"incid mon: {np.min(ts1)-t0} ~ {np.max(ts1)-t0}")
                     print(f"detector: start={ts0-t0}, force_synch={force_synch_trig}, exp = {exp}")
-                    raise
+                    ts1 += ts2[0]-ts1[0]
+                    print(f"using force_synch=0, incid mon: {np.min(ts1)-t0} ~ {np.max(ts1)-t0}")
+                    incid_data0 = integrate_mon(incid_data, ts1, ts0+force_synch_trig, exp, extend_mon_stream)
 
                 if plot_trigger:
                     plt.figure()
@@ -1209,6 +1089,9 @@ class h5xs():
         assume that the shape of the data is unchanged
         """
         
+        if self.read_only:
+            print("h5 file is read-only ...")
+            return
         if self.save_d1 is False:
             print("requested to save_d1s() but h5xs.save_d1 is False.")
             return
@@ -1217,16 +1100,16 @@ class h5xs():
             for sn in self.samples:
                 self.save_d1s(sn)
         
-        fh5 = self.fh5        
-        if "processed" not in list(lsh5(fh5[sn], top_only=True, silent=True)):
-            grp = fh5[sn].create_group("processed")
+        self.enable_write(True)        
+        if "processed" not in list(lsh5(self.fh5[sn], top_only=True, silent=True)):
+            grp = self.fh5[sn].create_group("processed")
         else:
-            grp = fh5[sn+'/processed']
+            grp = self.fh5[sn+'/processed']
             g0 = lsh5(grp, top_only=True, silent=True)[0]
             if grp[g0][0].shape[1]!=len(self.qgrid): # if grp[g0].value[0].shape[1]!=len(self.qgrid):
                 # new size for the data
-                del fh5[sn+'/processed']
-                grp = fh5[sn].create_group("processed")
+                del self.fh5[sn+'/processed']
+                grp = self.fh5[sn].create_group("processed")
         
         # these attributes are not necessarily available when save_d1s() is called
         if sn in list(self.attrs.keys()):
@@ -1251,7 +1134,7 @@ class h5xs():
             if (np.asarray(tvs)>0).any(): 
                 grp[k].attrs['trans'] = tvs
                 
-        fh5.flush()
+        self.enable_write(False)
 
     def average_d1s(self, samples=None, update_only=False, selection=None, filter_data=False, debug=False):
         """ if update_only is true: only work on samples that do not have "merged' data
@@ -1290,7 +1173,7 @@ class h5xs():
 
         if debug is True:
             t2 = time.time()
-            print("done, time lapsed: %.2f sec" % (t2-t1))
+            print("done, time elapsed: %.2f sec" % (t2-t1))
             
     def plot_d1s(self, sn, ax=None, offset=1.5, fontsize="large",
                     show_overlap=False, show_subtracted=False, show_subtraction=True):
@@ -1456,11 +1339,11 @@ class h5xs():
                         images[det.extension] = dset[gn][i*c_size:i*c_size+nframes]    
 
                     if N>1: # multi-processing, need to keep track of total number of active processes                    
-                        job = pool.map_async(proc_d1merge, [(images, sn, nframes, i*c_size, debug,
+                        job = pool.map_async(proc_merge1d, [(images, sn, nframes, i*c_size, debug,
                                                              detectors, self.qgrid, reft, save_1d, save_merged, dtype)])
                         jobs.append(job)
                     else: # serial processing
-                        [sn, fr1, data] = proc_d1merge((images, sn, nframes, i*c_size, debug, 
+                        [sn, fr1, data] = proc_merge1d((images, sn, nframes, i*c_size, debug, 
                                                         detectors, self.qgrid, reft, save_1d, save_merged, dtype)) 
                         results[sn][fr1] = data                
                 else: # len(s)==4
@@ -1470,11 +1353,11 @@ class h5xs():
                             gn = f'{self.det_name[det.extension]}'
                             images[det.extension] = dset[gn][j, i*c_size:i*c_size+nframes]
                         if N>1: # multi-processing, need to keep track of total number of active processes
-                            job = pool.map_async(proc_d1merge, [(images, sn, nframes, i*c_size+j*s[1], debug,
+                            job = pool.map_async(proc_merge1d, [(images, sn, nframes, i*c_size+j*s[1], debug,
                                                                  detectors, self.qgrid, reft, save_1d, save_merged, dtype)])
                             jobs.append(job)
                         else: # serial processing
-                            [sn, fr1, data] = proc_d1merge((images, sn, nframes, i*c_size+j*s[1], debug, 
+                            [sn, fr1, data] = proc_merge1d((images, sn, nframes, i*c_size+j*s[1], debug, 
                                                             detectors, self.qgrid, reft, save_1d, save_merged, dtype)) 
                             results[sn][fr1] = data                
 
@@ -1503,86 +1386,3 @@ class h5xs():
             t2 = time.time()
             print("done, time lapsed: %.2f sec" % (t2-t1))
 
-    def load_data0(self, update_only=False, 
-                   reft=-1, save_1d=False, save_merged=False, debug=False, N=8):
-        """ assume multiple samples, parallel-process by sample
-            if update_only is true, only create 1d data for new frames (empty self.d1s)
-        """
-        if debug is True:
-            print("start processing: load_data()")
-            t1 = time.time()
-        
-        fh5 = self.fh5
-        self.samples = lsh5(fh5, top_only=True, silent=(not debug))
-        
-        processes = []
-        queue_list = []
-        results = {}
-        for sn in self.samples:
-            if sn not in list(self.attrs.keys()):
-                self.attrs[sn] = {}
-            if 'buffer' in list(fh5[sn].attrs):
-                self.buffer_list[sn] = fh5[sn].attrs['buffer'].split('  ')
-            if update_only and sn in list(self.d1s.keys()):
-                self.load_d1s(sn)   # load processed data saved in the file
-                continue
-                                    
-            self.d1s[sn] = {}
-            results[sn] = {}
-            images = {}
-            for det in self.detectors:
-                ti = fh5["%s/primary/data/%s" % (sn, self.det_name[det.extension])][...]  # ].value
-                if len(ti.shape)>3:
-                    ti = ti.reshape(ti.shape[-3:])      # quirk of suitcase
-                images[det.extension] = ti
-            
-            n_total_frames = len(images[self.detectors[0].extension])
-            if N>1: # multi-processing
-                if n_total_frames<N*N/2:
-                    Np = 1
-                    c_size = N
-                else:
-                    Np = N
-                    c_size = int(n_total_frames/N)
-                for i in range(Np):
-                    if i==Np-1:
-                        nframes = n_total_frames - c_size*(Np-1)
-                    else:
-                        nframes = c_size
-                    que = mp.Queue()
-                    queue_list.append(que)
-                    th = mp.Process(target=proc_sample, 
-                                    args=(que, images, sn, nframes,
-                                          self.detectors, self.qgrid, reft, save_1d, save_merged, debug, i*c_size) )
-                    th.start()
-                    processes.append(th)
-            else: # serial processing
-                [sn, fr1, data] = proc_sample(None, images, sn, n_total_frames,
-                                              self.detectors, self.qgrid, reft, save_1d, save_merged, debug) 
-                self.d1s[sn] = data                
-
-        if N>1:             
-            for que in queue_list:
-                [sn, fr1, data] = que.get()
-                results[sn][fr1] = data
-                print("data received: sn=%s, fr1=%d" % (sn,fr1) )
-            for th in processes:
-                th.join()
-
-            for sn in self.samples:
-                if sn not in results.keys():
-                    continue
-                data = {}
-                frns = list(results[sn].keys())
-                frns.sort()
-                for k in results[sn][frns[0]].keys():
-                    data[k] = []
-                    for frn in frns:
-                        data[k].extend(results[sn][frn][k])
-                self.d1s[sn] = data
-        
-        self.save_d1s(debug=debug)
-        if debug is True:
-            t2 = time.time()
-            print("done, time lapsed: %.2f sec" % (t2-t1))
-            
