@@ -47,7 +47,7 @@ BEAM_SIZE_hH = 4
 # this is the global setting
 TRANS_MODE = trans_mode.from_waxs
 # this is the minimum intensity to be used for trans calculations
-WAXS_THRESH = 10
+WAXS_THRESH = 1.
 
 
 # this is the scaling factor for indivudual curves that belong to the same sample
@@ -121,7 +121,22 @@ class Data1d:
         
         if save_ave and isinstance(image, str):
             self.save(image + ".ave", debug=debug)     
+    
+    def water_peak(self, q_start=1.85, q_end=2.15, debug=False):
+        # for solution scattering, hopefully this reflect the intensity of water scattering
+        idx = (self.qgrid > q_start) & (self.qgrid < q_end)  # & (self.data>0.5*np.max(self.data))
+        if len(self.qgrid[idx]) < 5:
+            print("not enough data points under the water peak, consider using a different trans_mode.")
+            #raise Exception()
+
+        # trying to narrow down the peak range turns out to be a bad idea
+        # the width then could vary between datasets, creating artificial fluctuation in trans 
+        #idx1 = idx & (self.data >= 0.95*np.max(self.data[idx]))
         
+        if (self.data[idx]<WAXS_THRESH).all() and debug!='quiet':
+            print("the data points for water peak calculation are below WAXS_THRESH: ", 
+                  np.max(self.data[idx]), WAXS_THRESH)                
+        return np.sum(self.data[idx]),np.average(self.qgrid[idx])
 
     def set_trans(self, transMode=None, trans=-1, ref_trans=-1,
                   calc_water_peak=False, q_start=1.85, q_end=2.15, debug=False):
@@ -142,24 +157,10 @@ class Data1d:
             self.transMode = transMode
         if self.transMode==trans_mode.from_waxs or calc_water_peak:
             # get trans for the near the maximum in the WAXS data
-            # for solution scattering, hopefully this reflect the intensity of water scattering
-            idx = (self.qgrid > q_start) & (self.qgrid < q_end)  # & (self.data>0.5*np.max(self.data))
-            if len(self.qgrid[idx]) < 5:
-                print("not enough data points under the water peak, consider using a different trans_mode.")
-                #raise Exception()
-            
-            # trying to narrow down the peak range turns out to be a bad idea
-            # the width then could vary between datasets, creating artificial fluctuation in trans 
-            #idx1 = idx & (self.data >= 0.95*np.max(self.data[idx]))
-
-            if (self.data[idx]<WAXS_THRESH).all() and debug!='quiet':
-                print("the data points for trans calculation are below WAXS_THRESH: ", 
-                      np.max(self.data[idx]), WAXS_THRESH)                
-            self.trans_w = np.sum(self.data[idx])
+            self.trans_w,qavg = self.water_peak(q_start=q_start, q_end=q_end, debug=debug) 
             if self.transMode==trans_mode.from_waxs:
-                self.trans = self.trans_w 
-            qavg = np.average(self.qgrid[idx])
-            if self.trans_w<1.0:
+                self.trans = self.trans_w             
+            if self.trans_w<1.0:    # this need to be more configurable
                 print(f'caluclated trans is {self.trans_w}, setting it artifically to WAXS_THRESH.')
                 self.trans_w = WAXS_THRESH
             if debug==True:
@@ -174,8 +175,7 @@ class Data1d:
             self.comments += f"# transmitted beam intensity given externally: {trans}"
             self.trans_e = trans
             self.trans = trans
-            #if self.transMode==trans_mode.external:
-            #    self.trans = trans
+
 
         self.comments += ": %f \n" % self.trans
         if debug==True:
@@ -414,15 +414,13 @@ class Data1d:
                 
         return self
 
-    def merge(self, d1, qmax=-1, qmin=-1, fix_scale=-1, debug=False, create_new=False):
+    def merge(self, d1, debug=False, create_new=False):
         """
         combine the data in self and d1
         scale d1 intensity to match self
         self and d1 should have the same qgrid
 
-        if qmax or qmin <0
-        simply keep the WAXS data that is beyond qmax for the SAXS data
-        this is useful for utilizing WAXS to normalize intensity but keep SAXS data only
+        update 2022Apr: this function no longer scales data to match 
         """
 
         if debug==True:
@@ -431,78 +429,30 @@ class Data1d:
             print("merging data sets should have the same qgrid.")
             exit()
 
+        if debug==True:
+            print("merged set re-named %s." % self.label)
+
         # this gives the overlapping region
         idx = (self.data > 0) & (d1.data > 0)
-
         if len(self.qgrid[idx]) > 0:
-            qmin0 = min(d1.qgrid[idx])
-            qmax0 = max(self.qgrid[idx])
-            # merge SAXS/WAXS based on intensity in the overlapping region
-            if qmax0 < qmax:
-                qmax = qmax0
-            if qmin0 > qmin:
-                qmin = qmin0
-            idx = (self.qgrid > qmin) & (self.qgrid < qmax)
             # save the raw data in case needed, e.g. for ploting
             self.overlaps.append({'q_overlap': self.qgrid[idx],
                                   'raw_data1': self.data[idx],
                                   'raw_data2': d1.data[idx]})
-        else:
-            # no overlap
-            # simply stack WAXS data to the high q end of SAXS data
-            qmin = qmax = max(self.qgrid[self.data > 0])
-            self.overlaps.append({'q_overlap': np.empty(0),
-                                  'raw_data1': np.empty(0),
-                                  'raw_data2': np.empty(0)})
-
-        # idx = np.asarray([],dtype=int)
-
-        if len(self.qgrid[idx])==0:
-            if debug!='quiet':
-                print("data sets are not overlapping in the given q range.")
-            if fix_scale < 0:
-                fix_scale = 1
-                if debug!='quiet':
-                    print("forcing fix_scale=1.")
-        elif len(self.qgrid[idx]) < 5 and debug!='quiet':
-            print("too few overlapping points: %d" % len(self.qgrid[idx]))
-
-        if fix_scale > 0:
-            # For a given experimental configuration, the intensity normlization
-            # factor between the SAXS and WAXS should be well-defined. This factor
-            # can be determined using scattering data with siginificant intensity
-            # in the overlapping q-range and applied to all data collected in the
-            # same configuration.
-            sc = fix_scale
-        else:
-            sc = np.linalg.lstsq(np.asmatrix(self.data[idx]).T, np.asmatrix(d1.data[idx]).T)[0]
-            sc = np.trace(sc)
-
-        d1.data /= sc
-        d1.err /= sc
-        if len(self.qgrid[idx]) > 0:
-            if debug==True:
-                print("Scaled Overlaps by 1/%f" % sc)
-            self.overlaps[-1]['raw_data2'] /= sc
-            self.overlaps[-1]['sc'] = sc
-
-        self.label = common_name(self.label, d1.label)
-        if debug==True:
-            print("set2 scaled by 1/%f" % sc)
-            print("merged set re-named %s." % self.label)
-
-        if len(self.qgrid[idx]) > 0:
             # averaging using 1/err^2 as weight, maximum likelihood estimator 
             w1 = 1./(self.err[idx]*self.err[idx]) 
             w2 = 1./(d1.err[idx]*d1.err[idx]) 
             self.data[idx] = (w1*self.data[idx] + w2*d1.data[idx]) / (w1+w2)
             self.err[idx] = np.sqrt((w1/(w1+w2)*self.err[idx])**2 + (w2/(w1+w2)*d1.err[idx])**2)
-        self.data[self.qgrid >= qmax] = d1.data[self.qgrid >= qmax]
-        self.err[self.qgrid >= qmax] = d1.err[self.qgrid >= qmax]
+        
+        # where d1 has data 
+        idx1 = (d1.data > 0) & ~(self.data > 0)
+        self.data[idx1] = d1.data[idx1]
+        self.err[idx1] = d1.err[idx1]
 
-        self.comments += "# merged with the following set by matching intensity within (%.4f, %.4f)," % (qmin, qmax)
-        self.comments += " scaled by %f\n" % sc
-        self.comments += d1.comments.replace("# ", "## ")
+        qmin = self.qgrid[idx][0]
+        qmax = self.qgrid[idx][-1]
+        
 
     def plot_Guinier(self, qs=0, qe=10, rg=15, fix_qe=False, scale_wabs=-1,
                      ax=None, no_plot=False, fontsize="large"):
@@ -882,7 +832,7 @@ def analyze(d1, qstart, qend, fix_qe, qcutoff, dmax):
     plt.subplots_adjust(bottom=0.15, wspace=0.25)
 
     
-def estimate_scaling_factor(d1s, d1b, 
+def estimate_scaling_factor(d1s, d1b, sc=0.9,
                             q_min=0.5, q_max=3.5, smoothing_width=5, prec=4, s_thresh=1,
                             plot_data=False, ax=None, debug=False):
     """ Estimate the scaling factor needed to subtract buffer scattering d1b
@@ -908,7 +858,7 @@ def estimate_scaling_factor(d1s, d1b,
     except:
         return 1.0      # problems like incomplete data can cause smooth() to fail, no point to continue
     mq = d1s.qgrid[idx]
-    sc = 0.9
+    
     prec0 = 2
     std0 = np.log(md1s-md1b*sc).std()
     n,bins = np.histogram((md1s-md1b*sc)*mq)
